EnableExplicit

XIncludeFile "MPFR.pbi"

Define GMPLibrary.i

GMPLibrary = OpenLibrary(#PB_Any, "libgmp-10.dll")

If GMPLibrary
  PrototypeC   mpz_init ( *x.mpz)
  PrototypeC   mpz_clear ( *x.mpz)
  PrototypeC   gmp_randinit_default ( *state.gmp_randstate_t)
  PrototypeC   gmp_randseed_ui ( *state.gmp_randstate_t, seed.i)
  PrototypeC   gmp_randclear ( *state.gmp_randstate_t)
  
  Global mpz_init.i : mpz_init.mpz_init = GetFunction(GMPLibrary, "__gmpz_init" )
  Global mpz_clear.i : mpz_clear.mpz_clear = GetFunction(GMPLibrary, "__gmpz_clear" )
  Global gmp_randinit_default.i : gmp_randinit_default.gmp_randinit_default = GetFunction(GMPLibrary, "__gmp_randinit_default" )
  Global gmp_randseed_ui.i : gmp_randseed_ui.gmp_randseed_ui = GetFunction(GMPLibrary, "__gmp_randseed_ui" )
  Global gmp_randclear.i : gmp_randclear.gmp_randclear = GetFunction(GMPLibrary, "__gmp_randclear" )
Else
  Debug "GMP library not found"
  End
EndIf

Define String.s=Space(1024)
Define x.mpfr

Define expptr.i
Define base.l=10
Define ndigits.i=8
Define n.i

mpfr_init2(x,256)

;######################################
;Enter the line number you want to test
#LINE=4


CompilerIf #LINE=1
  mpfr_set_ui(x,2,#MPFR_RNDN)
  mpfr_log10(x,x,#MPFR_RNDN)
  mpfr_sprintf (String,"%57.52R*e", #MPFR_RNDN, x)
  Debug String
CompilerElseIf #LINE=2
  mpfr_const_log2 ( x, #MPFR_RNDN)
  mpfr_sprintf (String,"%57.52R*e", #MPFR_RNDN, x)
  Debug String
CompilerElseIf #LINE=3
  mpfr_const_pi ( x, #MPFR_RNDN)
  mpfr_sprintf (String,"%57.52R*e", #MPFR_RNDN, x)
  Debug String
CompilerElseIf #LINE=4
  mpfr_const_euler ( x, #MPFR_RNDN)
  mpfr_sprintf (String,"%57.52R*e", #MPFR_RNDN, x)
  Debug String
CompilerElseIf #LINE=5
  mpfr_const_catalan ( x, #MPFR_RNDN)
  mpfr_sprintf (String,"%57.52R*e", #MPFR_RNDN, x)
  Debug String
CompilerElseIf #LINE=6
  Debug PeekS(mpfr_get_version ())
CompilerElseIf #LINE=7
  Debug PeekS(mpfr_buildopt_tune_case ())
CompilerElseIf #LINE=8
  Debug PeekS(mpfr_print_rnd_mode (0))
CompilerElseIf #LINE=9
  mpfr_fac_ui ( x, 5, #MPFR_RNDN)
  Debug "function returns = "+PeekS(mpfr_get_str ( String, @expptr, base, ndigits, x, #MPFR_RNDN))
  Debug "string value = "+String+", exponent = "+expptr
  Debug "so the value = 0."+String+" * 10^"+Str(expptr)
CompilerElseIf #LINE=10
  OpenConsole()
  PrintN("show different ways to round a number")
  n = 3
  mpfr_zeta_ui ( x, n, #MPFR_RNDN)
  PrintN("Zeta("+Str(n)+") = ")
  mpfr_printf ("%28.21Rf", x) : PrintN("  round to nearest (default)")
  mpfr_printf ("%28.21RNf", x) : PrintN("  round to nearest (explicit)")
  mpfr_printf ("%28.21R*f", #MPFR_RNDZ, x) : PrintN("  round toward zero (explicit)")
  mpfr_printf ("%28.21R*e", #MPFR_RNDN, x) : PrintN("  round to nearest (explicit)")
  Print("press the return key to end. ")
  Input()
  CloseConsole()
CompilerElseIf #LINE=11
  If GMPLibrary
    OpenConsole()
    PrintN("show how to convert an mpfr number to mpz and print same using mpfr_printf")
    Define k.mpz
    mpz_init(k)
    n = 5
    mpfr_fac_ui ( x, n, #MPFR_RNDN)
    mpfr_get_z (k, x, #MPFR_RNDN)
    Print("Factorial(5) = ")
    mpfr_printf ("%Zd", k)
    PrintN("")  
    Print("press the return key to end. ")
    Input()
    CloseConsole()
    mpz_clear(k)
  EndIf
CompilerElseIf #LINE=12
  If GMPLibrary
    OpenConsole()
    PrintN(" mpfr random numbers") 
    Define rand_state.gmp_randstate_t
    Define rand_num2.mpfr
    Define ran_bnd2.mpfr
    Define r.i
    ;rand_bits.l=64
    
    mpfr_init(rand_num2)
    mpfr_init(ran_bnd2)
    
    gmp_randinit_default(rand_state)
    gmp_randseed_ui(rand_state, 1234567890)
    For r=1 To 10
      ;mpfr_urandomb(rand_num2, rand_state)
      ;mpfr_grandom(rand_num2, rand_num2, rand_state, #MPFR_RNDN)
      mpfr_urandom(rand_num2, rand_state, #MPFR_RNDN)
      String=Space(256)
      mpfr_sprintf(String,"%.33R*f", #MPFR_RNDN, rand_num2)
      PrintN(String)
    Next
    Print("press the return key to end. ")
    Input()
    mpfr_clear(ran_bnd2)
    mpfr_clear(rand_num2)
    gmp_randclear(rand_state)
  EndIf
CompilerEndIf

mpfr_clear(x)

End

; IDE Options = PureBasic 5.31 (Windows - x64)
; ExecutableFormat = Console
; CursorPosition = 37
; FirstLine = 11
; EnableXP